function [transx1, transx2, transx3, lam, exitflag]=boxcox3d(x1,x2,x3)
%Input: x1,x2,x3 i.e. the original data.
%Output: transx1, transx2, transx3 (i.e. the transformed data),
%"lam", which is the estimated BoxCox parameter 
%"exitflag", which informs us if convergence occured.

         logL=@(h) -likbox(x1,x2,x3,h); %this is minus the logL that depends on h (that is lambda)
         %the logL uses the likbox routine to calculate the value of logL
         %for a given h. i.e. it is a function of h, and will be used right
         %next to fminsearch for minimization:
         
         init=0;
         
         [lam , ~, exitflag]=fminsearch(logL,init); %minimize it with initial value 1.
         
         if lam==0 %which is never going to happen
             transx1=log(x1);
             transx2=log(x2);
             transx3=log(x3);
             
         else
             transx1=((x1.^lam)-1)./lam;
             transx2=((x2.^lam)-1)./lam;
             transx3=((x3.^lam)-1)./lam;
             
         end
             
end

function [out]=likbox(x1,x2,x3,h)
%This calculates the value of the logL for a given h. The result is "out".
%Remember that x1,x2 and x3 are the original data.
n1=length(x1);
n2=length(x2);
n3=length(x3);
if h==0
    xh1=log(x1);
    xh2=log(x2);
    xh3=log(x3);
else 
    xh1=((x1.^h)-1)./h;
    xh2=((x2.^h)-1)./h;
    xh3=((x3.^h)-1)./h;
    
end

s1=sqrt(sum((xh1-sum(xh1)/n1).^2)./n1);%note that here I use the transformed data xh1,xh2,xh3 and NOT x1,x2,x3.
s2=sqrt(sum((xh2-sum(xh2)/n2).^2)./n2);
s3=sqrt(sum((xh3-sum(xh3)/n3).^2)./n3);

out1=(h-1).*(sum(log(x1))+sum(log(x2))+sum(log(x3))); %note that here I use the original data, x1,x2,x3.
out2=-n1/2.*(1+log(2*pi*s1.^2))-n2/2.*(1+log(2*pi*s2.^2))-n3/2.*(1+log(2*pi*s3.^2));
out=out1+out2;


end

